@@ -1,9 +1,5 @@
-#include "config.h"
+#include "common.h"
 #include <curses.h>
-#if HAVE_GETOPT_H
-# include <getopt.h>
-#endif
-#include <libnjb.h>
 #include <limits.h>
 #include <signal.h>
 #include <stdio.h>
