@@ -1,10 +1,4 @@
-#include "config.h"
-#include <libnjb.h>
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#else
-#include <unistd.h>
-#endif
+#include "common.h"
 
 void usage (void);
 
